var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import util from 'util';
import * as dotenv from 'dotenv';
import * as logger from 'logger';
import exceptionalError from 'exceptionalhandler';
export { exceptionalError };
import * as sfAPI from './salesforceAPI.js';
import * as sfUtilities from './salesforceUtilities.js';
dotenv.config();
const isProduction = (process.env.NODE_ENV === "PRODUCTION");
export { logger };
import * as orderObject from './sfObjects/order.js';
import * as accountObject from './sfObjects/account.js';
import * as locationObject from './sfObjects/location.js';
import * as returnObject from './sfObjects/return.js';
import * as inventoryObject from './sfObjects/inventory.js';
let reloadAfterCreation = true;
if (!isProduction) {
    reloadAfterCreation = false;
}
const orderConstants = orderObject.objConstants;
//WRITE
function sendObjToSF(objectToWrite) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!objectToWrite.type) {
            throw new exceptionalError(`Invalid targetType provded to writeFile`);
        }
        if (!objectToWrite.writeable) {
            throw new exceptionalError(`Object type ${objectToWrite.type} is not writeable`);
        }
        let httpMethod = '';
        let endPoint = '';
        switch (objectToWrite.type) {
            case "location":
                endPoint = `${sfAPI.endPointWriteLocation}`;
                break;
            case "order":
                endPoint = `${sfAPI.endPointWriteOrder}`;
                break;
            case "return":
                endPoint = `${sfAPI.endPointWriteReturn}`;
                break;
        }
        let updateFile = false;
        if (objectToWrite.loadedFromSF) {
            if (!objectToWrite.details.id) {
                throw new exceptionalError("Can't update a field without an Id");
            }
            httpMethod = 'patch';
            endPoint = `${endPoint}${objectToWrite.details.id}`;
            updateFile = true;
        }
        else {
            httpMethod = 'post';
        }
        let write = null;
        let sfObj = objectToWrite.createSF(updateFile);
        try {
            write = yield sfAPI.performAPIReq(httpMethod, endPoint, 'body', sfObj);
        }
        catch (err) {
            inspectWriteError(err, objectToWrite);
        }
        if (write.code == 201) {
            if (reloadAfterCreation) {
                let reload;
                switch (objectToWrite.type) {
                    case "location":
                        reload = yield getBD2Locations({ id: write.newId });
                        break;
                    case "order":
                        reload = yield getBD2Orders({ id: write.newId });
                        break;
                    case "return":
                        reload = yield getBD2Returns(({ id: write.newId }));
                        break;
                }
                if (reload.length !== 1) {
                    throw new exceptionalError(`Got ${reload.length} returns for id ${write.newId}`);
                }
                return reload[0];
            }
            else {
                objectToWrite.details.id = write.newId;
                objectToWrite.loadedFromSF = true;
                return objectToWrite;
            }
        }
        else if (write.code == 204) {
            return objectToWrite;
        }
        else {
            throw `How did I end up here with code ${write.code}!?!?`;
        }
    });
}
export function writeFile(objectToWrite) {
    return __awaiter(this, void 0, void 0, function* () {
        return yield sendObjToSF(objectToWrite);
    });
}
function inspectWriteError(err, objectToWrite) {
    if (err instanceof exceptionalError) {
        if (!err.serverResponse || err.serverResponse.length === 0) {
            throw err;
        }
        else {
            if (Array.isArray(err.serverResponse)) {
                for (const errEntry of err.serverResponse) {
                    if (errEntry.errorCode.match(/INVALID_CROSS_REFERENCE_KEY|MALFORMED_ID/)) {
                        throw new exceptionalError(`Invalid ${objectToWrite.type} parent provided`);
                    }
                    if (errEntry.errorCode.match(/DUPLICATE_VALUE/)) {
                        let dupeName = errEntry.message.match(/value found: (.+?) ?duplicates/i);
                        let errMsg = '';
                        if (dupeName) {
                            errMsg = `SF rejected write due to duplicate value on field "${dupeName[1]}"`;
                        }
                        else {
                            errMsg = `SF rejected write due to duplicate value(s) on unspecified field(s)`;
                        }
                        throw new exceptionalError(errMsg);
                    }
                    if (errEntry.errorCode.match(/REQUIRED_FIELD_MISSING/)) {
                        throw new exceptionalError(`Missing fields: ${errEntry.fields.join(" | ")}`);
                    }
                }
            }
        }
    }
    throw err;
}
// READ
function getObjectByFilter(targetType, filters) {
    return __awaiter(this, void 0, void 0, function* () {
        let query = sfUtilities.queryGenerator(targetType, filters);
        const request = yield sfAPI.performAPIReq('get', sfAPI.endPointQueries, 'url', query);
        let returnVals = [];
        if (request.response.totalSize > 0) {
            for (const record of request.response.records) {
                let newObj = targetType.createNew();
                newObj.loadFromSF(record);
                returnVals.push(newObj);
            }
        }
        return returnVals;
    });
}
export function getBD2Accounts(filters) {
    return __awaiter(this, void 0, void 0, function* () {
        let returnVal = yield getObjectByFilter(accountObject, filters);
        return returnVal;
    });
}
export function getBD2Locations(filters) {
    return __awaiter(this, void 0, void 0, function* () {
        let returnVal = yield getObjectByFilter(locationObject, filters);
        return returnVal;
    });
}
export function getBD2Orders(filters) {
    return __awaiter(this, void 0, void 0, function* () {
        let returnVal = yield getObjectByFilter(orderObject, filters);
        return returnVal;
    });
}
export function getBD2OrdersByStatus(orderStatus, filters) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!filters.hasOwnProperty(orderObject.objConstants.niceNameOrderStatus)) {
            switch (orderStatus) {
                case "new":
                    filters[orderObject.objConstants.niceNameOrderStatus] = orderObject.objConstants.orderStatusOrdered;
                    break;
                case "unshipped":
                    filters[orderObject.objConstants.niceNameOrderStatus] = [orderObject.objConstants.orderStatusFulfilling, orderObject.objConstants.orderStatusPaused, orderObject.objConstants.orderStatusPartiallyShipped];
                    break;
                case "shipped":
                    filters[orderObject.objConstants.niceNameOrderStatus] = orderObject.objConstants.orderStatusShipped;
                    break;
                case "paused":
                    filters[orderObject.objConstants.niceNameOrderStatus] = orderObject.objConstants.orderStatusPaused;
                    break;
                default:
                    throw new exceptionalError(`Invalid status \"${orderStatus}\"`);
            }
        }
        return yield getBD2Orders(filters);
    });
}
export function getBD2Inventories(filters) {
    return __awaiter(this, void 0, void 0, function* () {
        let returnVal = yield getObjectByFilter(inventoryObject, filters);
        return returnVal;
    });
}
export function getBD2Returns(filters) {
    return __awaiter(this, void 0, void 0, function* () {
        let returnVal = yield getObjectByFilter(returnObject, filters);
        return returnVal;
    });
}
// Nice methods
export function createOrFindLocation(locationObj) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!locationObj || locationObj.type != "location") {
            throw `This method only accepts location objects`;
        }
        let locationExistsCheck = yield findDuplicateLocation(locationObj);
        if (locationExistsCheck) {
            let updated = false;
            for (const [existingKey, existingVal] of Object.entries(locationExistsCheck.details)) {
                if (Object.prototype.toString.call(existingVal) === '[object Object]') {
                    for (const [existingSubKey, existingSubVal] of Object.entries(existingVal)) {
                        //@ts-ignore
                        if (existingSubVal === null && locationObj.details[existingKey][existingSubKey] !== null) { //
                            //@ts-ignore
                            locationExistsCheck.details[existingKey][existingSubKey] = locationObj.details[existingKey][existingSubKey];
                            updated = true;
                        }
                    }
                }
                else {
                    //@ts-ignore
                    if (existingVal === null && locationObj.details[existingKey] !== null) {
                        //@ts-ignore
                        locationExistsCheck.details[existingKey] = locationObj.details[existingKey];
                        updated = true;
                    }
                }
            }
            if (updated) {
                return yield sendObjToSF(locationExistsCheck);
            }
            return locationExistsCheck;
        }
        return yield sendObjToSF(locationObj);
    });
}
function findDuplicateLocation(locationObj) {
    return __awaiter(this, void 0, void 0, function* () {
        logger.info(`Looking up duplicates for ${locationObj.details.address.streetAddress} ${locationObj.details.address.city} ${locationObj.details.address.province} ${locationObj.details.address.postalCode} ${locationObj.details.address.country}`);
        let newStreetAddress = `${locationObj.details.address.streetAddress}`;
        const duplicateSelectionFilter = sfUtilities.setDupeFiltersForObj(locationObj);
        if (!duplicateSelectionFilter) {
            return null;
        }
        let dupeCheck = yield getBD2Locations(duplicateSelectionFilter);
        if (dupeCheck.length === 0) {
            return null;
        }
        for (const possibleMatch of dupeCheck) {
            let checkStreet = possibleMatch.details.address.streetAddress;
            if (sfUtilities.compareStreets(newStreetAddress, checkStreet)) {
                return possibleMatch;
            }
        }
        return null;
    });
}
function validateNewOrder(orderObj, orderType, inventorySource, shippingService) {
    if (!orderType || !inventorySource || !shippingService) {
        throw new exceptionalError("Order type, source, and shipping service must be defined");
    }
    //@ts-ignore
    orderType = orderType.toLowerCase();
    //@ts-ignore
    inventorySource = inventorySource.toLowerCase();
    if (!orderObject.objConstants.validCreateOrderDetailsTypes.includes(orderType)) {
        throw new exceptionalError(`Invalid order type \"${orderType}\"`);
    }
    if (!orderObject.objConstants.validCreateOrderDetailsSources.includes(inventorySource)) {
        throw new exceptionalError(`Invalid inventory source \"${inventorySource}\"`);
    }
    if (!orderObject.objConstants.validShippingServices.includes(shippingService)) {
        throw new exceptionalError(`Invalid shipping service \"${shippingService}\"`);
    }
    orderObj.details.fulfillment.shippingService = shippingService;
    if (!orderObj.details.fulfillment.qtyRequested || orderObj.details.fulfillment.qtyRequested === 0) {
        throw new exceptionalError(`Invalid Quantity ${orderObj.details.fulfillment.qtyRequested}`);
    }
    const orderStatus = orderObj.details.fulfillment.orderStatus;
    if (orderStatus && orderStatus.match(/shipped/i)) {
        if (!orderObj.details.fulfillment.shippingTrackingCarrier || !orderObj.details.fulfillment.shippingTrackingNumber || orderObj.details.fulfillment.shippingTrackingCarrier === "" || orderObj.details.fulfillment.shippingTrackingNumber === "") {
            throw new exceptionalError("Orders require tracking to be created in the shipped or partially shipped state");
        }
    }
}
function processNewFields(orderObj, orderDeets) {
    if (orderDeets.shopifyOrder) {
        if (orderDeets.source === "bulk") {
            orderObj.details.orderDetails.orderType = orderConstants.orderTypeBulkOrder;
            orderDeets.invoiceStatus = orderConstants.invoiceStatusNA;
        }
        else {
            orderObj.details.orderDetails.orderType = orderConstants.orderTypeShopifyOrder;
            orderDeets.invoiceStatus = orderConstants.invoiceStatusShopify;
        }
    }
    else {
        switch (orderDeets.source) {
            case "default":
                orderDeets.invoiceStatus = orderConstants.invoiceStatusNotSent;
                orderObj.details.orderDetails.orderType = orderConstants.orderTypeInvoicedOrder;
                break;
            case "bulk":
                orderDeets.invoiceStatus = orderConstants.invoiceStatusNA;
                orderObj.details.orderDetails.orderType = orderConstants.orderTypeBulkOrder;
                break;
            default:
                throw new exceptionalError(`Invalid new combo: ${orderDeets.type} & ${orderDeets.source}`);
        }
    }
    return orderDeets;
}
function processReplacementFields(orderObj, orderDeets) {
    orderDeets.invoiceStatus = orderConstants.invoiceStatusNA;
    switch (orderDeets.source) {
        case "default":
            orderObj.details.orderDetails.orderType = orderConstants.orderTypeReplacement;
            break;
        case "bulk":
            orderObj.details.orderDetails.orderType = orderConstants.orderTypeBulkReplacement;
            break;
        default:
            throw new exceptionalError(`Invalid replacement combo: ${orderDeets.type} & ${orderDeets.source}`);
    }
    return orderDeets;
}
function processSampleFields(orderObj, orderDeets) {
    orderDeets.invoiceStatus = orderConstants.invoiceStatusNA;
    switch (orderDeets.source) {
        case "default":
            orderObj.details.orderDetails.orderType = orderConstants.orderTypeSample;
            break;
        case "bulk":
            orderObj.details.orderDetails.orderType = orderConstants.orderTypeBulkSample;
            break;
        default:
            throw new exceptionalError(`Invalid sample combo: ${orderDeets.type} & ${orderDeets.source}`);
    }
    return orderDeets;
}
function setOrderTypes(orderObj, orderDeets) {
    sfUtilities.validateCash(orderObj, orderDeets);
    switch (orderDeets.type) {
        case "new":
            orderDeets = processNewFields(orderObj, orderDeets);
            break;
        case "replacement":
            orderDeets = processReplacementFields(orderObj, orderDeets);
            break;
        case "sample":
            orderDeets = processSampleFields(orderObj, orderDeets);
            break;
        default:
            throw new exceptionalError("Never should have got here!");
    }
    return orderDeets;
}
function setDefaultValues(orderObj) {
    if (!orderObj.details.fulfillment.orderStatus) {
        orderObj.details.fulfillment.orderStatus = orderObject.objConstants.orderStatusOrdered;
    }
    if (!orderObj.details.fulfillment.qtyFulfilled) {
        orderObj.details.fulfillment.qtyFulfilled = 0;
    }
    return orderObj;
}
function validateTargetBulkSource(inventoryId, qtyRequested, type) {
    return __awaiter(this, void 0, void 0, function* () {
        let targetInventory = null;
        let inventoryCheck = yield getBD2Inventories({ id: inventoryId });
        if (inventoryCheck.length === 1) {
            targetInventory = inventoryCheck[0];
            logger.info(`Bulk order for ${qtyRequested} units - checking for availability in ${targetInventory.details.id} - ${targetInventory.details.qty.qtyAvailable} | permissions new: ${targetInventory.details.permittedUse.allowNew} replacement: ${targetInventory.details.permittedUse.allowReplacement}`);
            const bulkValidated = (type === "new" && targetInventory.details.qty.qtyAvailable >= qtyRequested && targetInventory.details.permittedUse.allowNew) || (type === "replacement" && targetInventory.details.qty.qtyAvailable >= qtyRequested && targetInventory.details.permittedUse.allowReplacement);
            if (bulkValidated) {
                logger.info(`Target inventory is valid`);
                return targetInventory;
            }
        }
        else {
            throw new exceptionalError(`No inventory found for ID ${inventoryId}`);
        }
        return null;
    });
}
function getAllInventoriesForParent(accountId, qtyRequested, type) {
    return __awaiter(this, void 0, void 0, function* () {
        let targetInventory = null;
        logger.info(`Looking for a valid inventory source`);
        let backupInventoryFilters = {
            accountId: accountId,
            qtyAvailable: `>=${qtyRequested}`
        };
        if (type === "new" || type === "sample") {
            backupInventoryFilters.allowNew = true;
        }
        else if (type === "replacement") {
            backupInventoryFilters.allowReplacement = true;
        }
        else {
            throw new exceptionalError(`How did we get here with type "${type}"`);
        }
        let allInventories = yield getBD2Inventories(backupInventoryFilters);
        if (allInventories.length > 0) {
            logger.info(`Found ${allInventories[0].details.id}`);
            return allInventories[0];
        }
        logger.info(`None found`);
        return null;
    });
}
function checkBulkAvailability(orderObj, orderDeets, parentAccount) {
    return __awaiter(this, void 0, void 0, function* () {
        let targetInventory = null;
        const inventoryValSpecified = !!orderObj.details.fulfillment.inventoryId;
        if (inventoryValSpecified) {
            targetInventory = yield validateTargetBulkSource(orderObj.details.fulfillment.inventoryId, orderObj.details.fulfillment.qtyRequested, orderDeets.type);
            if (!targetInventory) {
                orderDeets.notes.push(`Order specified bulk inventory ${orderObj.details.fulfillment.inventoryId} which was not available or valid`);
            }
        }
        if (!targetInventory) {
            targetInventory = yield getAllInventoriesForParent(orderObj.details.accountId, orderObj.details.fulfillment.qtyRequested, orderDeets.type);
            if (targetInventory) {
                orderDeets.notes.push(`Assigned order to inventory ID ${targetInventory.details.id}`);
            }
        }
        if (targetInventory) {
            orderDeets.invoiceStatus = orderConstants.invoiceStatusNA;
            if (targetInventory.details.shopifyVariantId) {
                if (orderObj.details.fulfillment.shippingService === orderConstants.shipServiceFedEx) {
                    orderObj.details.fulfillment.shippingService = orderConstants.shipServiceStandard;
                }
            }
            else {
                orderObj.details.fulfillment.shippingService = orderConstants.shipServiceFedEx;
            }
        }
        return targetInventory;
    });
}
function checkBulkFallbackInventory(orderObj, orderDeets, parentAccount) {
    return __awaiter(this, void 0, void 0, function* () {
        if (orderDeets.type === 'replacement' || parentAccount.details.options.bulkFallbackPermitted) {
            logger.info(`Falling back to RSL fulfillment`);
            if (!orderObj.details.fulfillment.inventoryId) {
                orderDeets.notes.push("Original request was bulk with no target inventory set - fallback to RSL");
            }
            orderObj.details.fulfillment.inventoryId = null;
            if (orderObj.details.fulfillment.shippingService === orderConstants.shipServiceFedEx) {
                orderObj.details.fulfillment.shippingService = orderConstants.shipServiceStandard;
            }
            ;
            if (orderDeets.type === 'replacement') {
                orderObj.details.orderDetails.orderType = orderConstants.orderTypeReplacement;
            }
            else {
                if (orderDeets.type === "new") {
                    if (orderDeets.shopifyOrder) {
                        orderObj.details.orderDetails.orderType = orderConstants.orderTypeShopifyOrder;
                        orderDeets.invoiceStatus = orderConstants.invoiceStatusNA;
                    }
                    else {
                        orderDeets.invoiceStatus = orderConstants.invoiceStatusNotSent;
                        orderObj.details.orderDetails.orderType = orderConstants.orderTypeInvoicedOrder;
                    }
                }
                else {
                    orderObj.details.orderDetails.orderType = orderConstants.orderTypeSample;
                }
            }
        }
        else {
            logger.info(util.inspect(orderObj.details));
            throw new exceptionalError(`Unable to assign inventory for order ${orderObj.details.externalOrderNumber} - order was not created`);
        }
    });
}
export function createNewOrder(orderObj, orderType, inventorySource, shippingService = orderConstants.shipServiceStandard, isShopifyOrder = false) {
    return __awaiter(this, void 0, void 0, function* () {
        validateNewOrder(orderObj, orderType, inventorySource, shippingService);
        let orderDeets = {
            type: orderType,
            source: inventorySource,
            notes: [],
            shopifyOrder: isShopifyOrder,
            isFromBulk: (inventorySource.toLowerCase() === "bulk"),
            manualInvoiceStatus: (!!orderObj.details.invoicing.invoiceStatus),
            invoiceStatus: orderObj.details.invoicing.invoiceStatus
        };
        orderDeets = setOrderTypes(orderObj, orderDeets);
        orderObj = setDefaultValues(orderObj);
        if (orderDeets.isFromBulk) {
            let parentAccountReq = yield getBD2Accounts({ id: orderObj.details.accountId });
            if (parentAccountReq.length !== 1) {
                throw new exceptionalError(`Invalid parent ID ${orderObj.details.accountId}`);
            }
            let parentAccount = parentAccountReq[0];
            let targetInventory = yield checkBulkAvailability(orderObj, orderDeets, parentAccount);
            if (targetInventory) {
                orderObj.details.fulfillment.inventoryId = targetInventory.details.id;
            }
            else {
                yield checkBulkFallbackInventory(orderObj, orderDeets, parentAccount);
            }
        }
        else {
            orderObj.details.fulfillment.inventoryId = null;
        }
        if (!orderDeets.manualInvoiceStatus) {
            orderObj.details.invoicing.invoiceStatus = orderDeets.invoiceStatus;
        }
        if (orderDeets.notes.length > 0) {
            orderObj.details.notes = orderDeets.notes.join("\n");
        }
        let writeObj = yield sendObjToSF(orderObj);
        return writeObj;
    });
}
