var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as dotenv from 'dotenv';
dotenv.config();
import { logger } from './salesforce.js';
import { exceptionalError } from './salesforce.js';
import util from 'util';
export const endPointQueries = '/services/data/v57.0/query';
export const endPointAuth = '/services/oauth2/token';
export const endPointWriteLocation = '/services/data/v57.0/sobjects/BD2_Location__c/';
export const endPointWriteOrder = '/services/data/v57.0/sobjects/BD2_Order__c/';
export const endPointWriteReturn = '/services/data/v57.0/sobjects/BD2_Return__c/';
export let sfBaseUrl;
export let sfKey;
export let sfSecret;
export let sfUsername;
export let sfPassword;
if (process.env.NODE_ENV === 'PRODUCTION') {
    sfBaseUrl = process.env.PROD_SALESFORCE_BASE_URL;
    sfKey = process.env.PROD_SALESFORCE_PROD_KEY;
    sfSecret = process.env.PROD_SALESFORCE_SECRET_KEY;
    sfUsername = process.env.PROD_SALESFORCE_LOGIN;
    sfPassword = process.env.PROD_SALESFORCE_PW;
}
else {
    sfBaseUrl = process.env.DEV_SALESFORCE_BASE_URL;
    sfKey = process.env.DEV_SALESFORCE_PROD_KEY;
    sfSecret = process.env.DEV_SALESFORCE_SECRET_KEY;
    sfUsername = process.env.DEV_SALESFORCE_LOGIN;
    sfPassword = process.env.DEV_SALESFORCE_PW;
}
export let token = null;
export let connected = false;
export let tokenRenewal = null;
// REQUEST
function validateApiCall(isTokenReq, endPoint) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!connected) {
            if (!isTokenReq) {
                yield connect();
                return true;
            }
        }
        if (!isTokenReq) {
            yield checkTokenExpiration();
            return true;
        }
        if (!endPoint) {
            throw new exceptionalError(`No endpoint URL specified`);
        }
    });
}
function setApiMethod(reqType, reqOpts) {
    switch (reqType) {
        case "get":
            reqOpts['method'] = 'GET';
            break;
        case "post":
            reqOpts['method'] = 'POST';
            reqOpts['headers']['Content-Type'] = 'application/json';
            break;
        case "patch":
            reqOpts['method'] = 'PATCH';
            reqOpts['headers']['Content-Type'] = 'application/json';
            break;
        default:
            throw new exceptionalError(`Unknown req type ${reqType}`);
    }
    return reqOpts;
}
function setApiParams(apiReqParams, argType, args, isTokenReq) {
    switch (argType) {
        case "body":
            apiReqParams.opts['body'] = JSON.stringify(args);
            break;
        case "url":
            let urlArgsString = "";
            if (typeof (args) === 'string') {
                urlArgsString = args;
            }
            else {
                let urlParamsArr = Object.entries(args).map(([k, v]) => (`${k}=${v}`));
                urlArgsString = urlParamsArr.join("&");
            }
            apiReqParams.urlParams = `?${urlArgsString}`;
            break;
        default:
            throw new exceptionalError(`Unknown arg type \"${argType}\"`);
    }
    if (!isTokenReq && !token) {
        throw new exceptionalError(`Tried to make a query without the token set`);
    }
    else {
        apiReqParams.opts.headers['Authorization'] = `Bearer ${token}`;
    }
    return apiReqParams;
}
export function performAPIReq(reqType, endPoint, argType, args) {
    return __awaiter(this, void 0, void 0, function* () {
        const isTokenReq = !(!endPoint.match(/token/));
        yield validateApiCall(isTokenReq, endPoint);
        let apiReqParams = {
            url: `${sfBaseUrl}${endPoint}`,
            urlParams: "",
            opts: {
                headers: {}
            }
        };
        apiReqParams.opts = setApiMethod(reqType, apiReqParams.opts);
        apiReqParams = setApiParams(apiReqParams, argType, args, isTokenReq);
        let reqUrl = `${apiReqParams.url}${apiReqParams.urlParams}`;
        reqUrl = reqUrl.replace(/%{2,}/g, '%').replace(/%/g, '%25');
        const rawResp = yield fetch(reqUrl, apiReqParams.opts);
        return yield handleResp(rawResp, isTokenReq, apiReqParams, reqUrl);
    });
}
// RESPONSE
function handleResp(respObj, isTokenReq, apiReqParams, reqUrl) {
    return __awaiter(this, void 0, void 0, function* () {
        logger.info(`SF API: ${reqUrl} | ${apiReqParams.opts.method} | Returned: ${respObj.status}`);
        switch (respObj.status) {
            case 200:
                return yield parseGet(respObj, isTokenReq, apiReqParams);
                break;
            case 201:
                return yield parseCreate(respObj);
                break;
            case 204:
                return { code: 204, response: { updated: true } };
                break;
            case 400:
                yield handleBadResp(respObj, reqUrl);
                break;
            case 401:
                yield handleBadResp(respObj, reqUrl);
                break;
            default:
                yield handleBadResp(respObj, reqUrl);
        }
        throw new exceptionalError(`Not sure how we got here with code ${respObj.status}`);
    });
}
function parseGet(respObj, isTokenReq, apiReqParams) {
    return __awaiter(this, void 0, void 0, function* () {
        let jsonResp = yield respObj.json();
        if (jsonResp.hasOwnProperty("nextRecordsUrl")) {
            let comboResponse = {
                "totalSize": jsonResp.totalSize,
                "done": false,
                "records": []
            };
            comboResponse.records = comboResponse.records.concat(jsonResp.records);
            while (jsonResp.hasOwnProperty("nextRecordsUrl")) {
                let newReq = yield fetch(`${sfBaseUrl}${jsonResp.nextRecordsUrl}`, apiReqParams.opts);
                jsonResp = yield newReq.json();
                comboResponse.records = comboResponse.records.concat(jsonResp.records);
            }
            jsonResp = comboResponse;
        }
        if (!isTokenReq) {
            if (!jsonResp.hasOwnProperty('totalSize')) {
                logger.info(jsonResp);
                throw new exceptionalError(`Response body didnt contain the totalSize field`);
            }
            if (!jsonResp.hasOwnProperty('records')) {
                logger.info(jsonResp);
                throw new exceptionalError(`Response body didnt contain the records array`);
            }
            if (jsonResp.records.length !== jsonResp.totalSize) {
                logger.info(jsonResp);
                throw new exceptionalError(`SF Req - totalsize ${jsonResp.totalSize} != records length ${jsonResp.records.length}`);
            }
        }
        return {
            code: respObj.status,
            response: jsonResp
        };
    });
}
function parseCreate(respObj) {
    return __awaiter(this, void 0, void 0, function* () {
        let jsonResp = yield respObj.json();
        if (!jsonResp.hasOwnProperty('id')) {
            logger.info(util.inspect(jsonResp));
            throw new exceptionalError(`No id field found on body of created object`);
        }
        return { code: respObj.status, newId: jsonResp.id };
    });
}
function handleBadResp(respObj, reqUrl) {
    return __awaiter(this, void 0, void 0, function* () {
        let errors = [];
        let errJson = null;
        if (respObj.body) {
            let respText = yield respObj.text();
            if (respText.match(/<td>.+?<\/td>/)) {
                yield logger.info(respText);
                yield logger.info('AN ERROR HAPPENED!!!');
                throw new exceptionalError('Ended up getting HTML back?');
            }
            try {
                errJson = JSON.parse(respText);
            }
            catch (e) { }
            if (!!errJson) {
                if (Array.isArray(errJson) && errJson.length > 0) {
                    for (const errEntry of errJson) {
                        let err = getErrorText(errEntry);
                        if (err) {
                            errors.push(err);
                        }
                    }
                }
                else {
                    let err = getErrorText(errJson);
                    if (err) {
                        errors.push(err);
                    }
                }
            }
            else {
                errors.push(respText);
            }
        }
        let errString = `Error getting ${reqUrl} - returned ${respObj.status}`;
        if (errors.length > 0) {
            errString = `${errString}\n  ${errors.join("\n  ")}`;
        }
        logger.info(util.inspect(respObj));
        let exc = new exceptionalError(errString);
        exc.reqUrl = respObj.url;
        exc.serverResponse = errJson;
        throw exc;
    });
}
function getErrorText(errEntry) {
    let err = ``;
    if (errEntry.hasOwnProperty("errorCode")) {
        err = `Error: ${errEntry.errorCode}`;
    }
    else if (errEntry.hasOwnProperty("error")) {
        err = `Error: ${errEntry.error}`;
    }
    if (errEntry.hasOwnProperty("message")) {
        err = `${err} - ${errEntry.message}`;
    }
    else if (errEntry.hasOwnProperty("error_description")) {
        err = `${err} - ${errEntry.error_description}`;
    }
    if (errEntry.hasOwnProperty("fields") && Array.isArray(errEntry.fields) && errEntry.length > 0) {
        err = `${err}\n\t-> ${errEntry.fields.join("\n\t-> ")}`;
    }
    if (err === '') {
        return null;
    }
    return err;
}
// SESSION CONTROL
export function connect() {
    return __awaiter(this, void 0, void 0, function* () {
        if (!connected) {
            token = yield getToken();
            connected = true;
            tokenRenewal = Number(new Date()) + 20 * 60 * 1000;
            return true;
        }
        else {
            logger.warn(`Tried to call salesforcer connect() with connected set to true`);
            return false;
        }
    });
}
export function disconnect() {
    token = null;
    connected = false;
    tokenRenewal = null;
}
function getToken() {
    return __awaiter(this, void 0, void 0, function* () {
        logger.info(`Requesting token`);
        const authArgs = {
            grant_type: 'password',
            client_id: sfKey,
            client_secret: sfSecret,
            username: sfUsername,
            password: sfPassword
        };
        let tokenResp = yield performAPIReq('post', endPointAuth, 'url', authArgs);
        if (!tokenResp.response || !tokenResp.response.access_token) {
            logger.info(util.inspect(tokenResp));
            throw new exceptionalError('Oauth Response was missing the token object');
        }
        logger.info(`Got token ${tokenResp.response.access_token}`);
        return tokenResp.response.access_token;
    });
}
function checkTokenExpiration() {
    return __awaiter(this, void 0, void 0, function* () {
        const oneMinute = 60 * 1000;
        if (tokenRenewal) {
            const diffTime = tokenRenewal - Number(new Date());
            if (diffTime < oneMinute) {
                logger.info(`Expiring token and asking for a new one (${Math.round(diffTime / 1000)} seconds remaining)`);
                disconnect();
                yield connect();
            }
        }
    });
}
// Only for testing
export function overrideTokenTime(tokenTime) {
    if (process.env.NODE_ENV === 'PRODUCTION') {
        throw `This isnt meant to be used outside testing!`;
    }
    logger.warn(`Overrode tokenRenewal from ${tokenRenewal} TO ${tokenTime}`);
    tokenRenewal = tokenTime;
}
