var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import chai from 'chai';
import chaiAsPromised from "chai-as-promised";
import fetchMock from 'fetch-mock';
import * as SfAPI from '../salesforceAPI.js';
import * as SalesForcer from '../salesforce.js';
import * as testAccountCases from './testDetails/salesforce.cases.accounts.js';
import * as testAuthCases from './testDetails/salesforce.cases.auth.js';
import * as testOrderCases from './testDetails/salesforce.cases.orders.js';
const expect = chai.expect;
chai.use(chaiAsPromised);
before(() => __awaiter(void 0, void 0, void 0, function* () {
    SfAPI.disconnect();
    fetchMock.reset();
}));
describe('Test the SalesForce API', () => __awaiter(void 0, void 0, void 0, function* () {
    context('Testing basic error handling', () => __awaiter(void 0, void 0, void 0, function* () {
        let currentTestCase = testAuthCases.apiMockCases.tokenAuthFailure;
        before(() => __awaiter(void 0, void 0, void 0, function* () {
            fetchMock.post(currentTestCase.req.url, { status: currentTestCase.req.code, body: currentTestCase.req.body });
        }));
        it('Loads defaults properly', () => __awaiter(void 0, void 0, void 0, function* () {
            expect(SfAPI.connected).to.be.false;
            expect(SfAPI.token).to.be.null;
        }));
        it('Throws an exception on failure to authenticate', () => __awaiter(void 0, void 0, void 0, function* () {
            yield expect(SfAPI.connect()).to.be.rejectedWith(currentTestCase.resp);
        }));
    }));
    context('Test Authorization', () => __awaiter(void 0, void 0, void 0, function* () {
        let missingTestCase = testAuthCases.apiMockCases.missingTokenResp;
        it('Throws an exception if it doesnt get a token', () => __awaiter(void 0, void 0, void 0, function* () {
            fetchMock.restore();
            fetchMock.post(missingTestCase.req.url, { status: missingTestCase.req.code, body: missingTestCase.req.body });
            yield expect(SfAPI.connect()).to.be.rejectedWith(missingTestCase.resp);
        }));
        let currentTestCase = testAuthCases.apiMockCases.goodTokenReq;
        it('Gets a token', () => __awaiter(void 0, void 0, void 0, function* () {
            fetchMock.restore();
            fetchMock.post(currentTestCase.req.url, { status: currentTestCase.req.code, body: currentTestCase.req.body });
            yield expect(SfAPI.connect()).to.eventually.equal(true);
            expect(SfAPI.token, `Expected ${currentTestCase.resp} but got ${SfAPI.token}`).to.equal(currentTestCase.resp);
            expect(SfAPI.connected, `Expected true but got ${SfAPI.connected}`).to.equal(true);
            expect(SfAPI.tokenRenewal).not.to.be.null;
            const acceptableOffsetRange = 1 * 1000;
            const now = new Date();
            const offsetTime = Number(now) + (20 * 60 * 1000);
            const offsetDifference = Math.abs(offsetTime - SfAPI.tokenRenewal);
            expect(offsetDifference).to.be.lessThan(acceptableOffsetRange);
        }));
        it('Shares the token across instances', () => __awaiter(void 0, void 0, void 0, function* () {
            expect(SfAPI.token).to.equal(SfAPI.token);
            expect(SfAPI.connected).to.equal(SfAPI.connected);
        }));
        it('Doesnt try to reconnect when connect is called twice', () => __awaiter(void 0, void 0, void 0, function* () {
            fetchMock.restore();
            let newToken = testAuthCases.apiMockCases.newTokenCheck;
            fetchMock.post(newToken.req.url, { status: newToken.req.code, body: newToken.req.body });
            expect(SfAPI.token, `Pre-connect expected: ${newToken.resp} but got ${SfAPI.token}`).to.equal(newToken.resp);
            yield expect(SfAPI.connect()).to.eventually.equal(false);
            expect(SfAPI.token, `Post-connect expected: ${newToken.resp} but got ${SfAPI.token}`).to.equal(newToken.resp);
        }));
        it('Reconnects when the token is 1 minute from expiring', () => __awaiter(void 0, void 0, void 0, function* () {
            fetchMock.restore();
            let newToken = testAuthCases.apiMockCases.newTokenCheck;
            fetchMock.post(newToken.req.url, { status: newToken.req.code, body: newToken.req.body });
            fetchMock.get(testAccountCases.getAccountNoMatch.req.url, testAccountCases.getAccountNoMatch.req.body);
            expect(SfAPI.connected).to.be.true;
            yield expect(SalesForcer.getBD2Accounts({ accountName: 'Bob Of Mainframe' })).not.to.be.rejected;
            expect(SfAPI.token, `Token should not have changed - expected: ${newToken.resp} but got ${SfAPI.token}`).to.equal(newToken.resp);
            SfAPI.overrideTokenTime(Number(new Date()) + 30 * 1000);
            yield expect(SalesForcer.getBD2Accounts({ accountName: 'Bob Of Mainframe' })).not.to.be.rejected;
            expect(SfAPI.token, `Token should have changed - expected: ${newToken.req.body.access_token} but got ${SfAPI.token}`).to.equal(newToken.req.body.access_token);
        }));
    }));
    context('Pagination works..', () => __awaiter(void 0, void 0, void 0, function* () {
        before(() => __awaiter(void 0, void 0, void 0, function* () {
            fetchMock.restore();
            fetchMock.get(testOrderCases.paginationTestCase.responseOne.url, testOrderCases.paginationTestCase.responseOne.body);
            fetchMock.get(testOrderCases.paginationTestCase.responseTwo.url, testOrderCases.paginationTestCase.responseTwo.body);
            fetchMock.get(testOrderCases.paginationTestCase.responseThree.url, testOrderCases.paginationTestCase.responseThree.body);
        }));
        it('Retrieves the full paginated response', () => __awaiter(void 0, void 0, void 0, function* () {
            let paginatedResponse = yield SalesForcer.getBD2OrdersByStatus("new", { accountId: "paginationTest" });
            expect(paginatedResponse.map((a) => (a.details))).to.deep.equal(testOrderCases.paginationTestCase.expectedResults);
        }));
    }));
}));
