var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import chai from 'chai';
import chaiAsPromised from "chai-as-promised";
import fetchMock from 'fetch-mock';
import * as testAccountCases from './testDetails/salesforce.cases.accounts.js';
import * as testAuthCases from './testDetails/salesforce.cases.auth.js';
import * as testOrderCases from './testDetails/salesforce.cases.orders.js';
import * as testLocationCases from './testDetails/salesforce.cases.locations.js';
import * as testInventoryCases from './testDetails/salesforce.cases.inventory.js';
import * as SalesForcer from '../salesforce.js';
import * as SfAPI from '../salesforceAPI.js';
import * as orderObject from '../sfObjects/order.js';
import * as locationObj from '../sfObjects/location.js';
const expect = chai.expect;
chai.use(chaiAsPromised);
describe('Test the SalesForcer Order Level Functions', () => __awaiter(void 0, void 0, void 0, function* () {
    before(() => __awaiter(void 0, void 0, void 0, function* () {
        SfAPI.disconnect();
        fetchMock.restore();
        let authMock = testAuthCases.apiMockCases.goodTokenReq;
        fetchMock.post(authMock.req.url, { status: authMock.req.code, body: authMock.req.body });
        yield SfAPI.connect();
        fetchMock.restore();
    }));
    context('Test order search by status (paused/ordered/etc)', () => __awaiter(void 0, void 0, void 0, function* () {
        before(() => __awaiter(void 0, void 0, void 0, function* () {
            fetchMock.restore();
            fetchMock.get(testOrderCases.getNewOrdersSuccessful.req.url, testOrderCases.getNewOrdersSuccessful.req.body);
            fetchMock.get(testOrderCases.getUnshippedOrdersSuccessful.req.url, (url, opts) => __awaiter(void 0, void 0, void 0, function* () {
                return testOrderCases.checkHeaderToken(opts, testOrderCases.getUnshippedOrdersSuccessful.req.body, testAuthCases.apiMockCases.goodTokenReq.resp);
            }));
            fetchMock.get(testOrderCases.noOrdersFound.req.url, testOrderCases.noOrdersFound.req.body);
            fetchMock.get(testOrderCases.getOrderInternalId.req.url, testOrderCases.getOrderInternalId.req.body);
            fetchMock.get(testOrderCases.getOrderExternalId.req.url, testOrderCases.getOrderExternalId.req.body);
            fetchMock.get(testOrderCases.getInternalIdDoesntExist.req.url, testOrderCases.getInternalIdDoesntExist.req.body);
        }));
        it('Returns an empty array for no orders found', () => __awaiter(void 0, void 0, void 0, function* () {
            let noOrders = yield SalesForcer.getBD2OrdersByStatus("paused", {});
            expect(noOrders).to.be.empty;
        }));
        it('Retrieves all orders by order status (for all locations/accounts)', () => __awaiter(void 0, void 0, void 0, function* () {
            let unshippedOrders = yield SalesForcer.getBD2OrdersByStatus("unshipped", {});
            expect(unshippedOrders.map((a) => (a.details))).to.deep.equal(testOrderCases.getUnshippedOrdersSuccessful.resp);
            let newOrders = yield SalesForcer.getBD2OrdersByStatus("new", {});
            expect(newOrders.map((a) => (a.details))).to.deep.equal(testOrderCases.getNewOrdersSuccessful.resp);
        }));
        it('Throws an exception for invalid order statuses', () => __awaiter(void 0, void 0, void 0, function* () {
            // @ts-ignore
            yield expect(SalesForcer.getBD2OrdersByStatus("asdadsa", {})).to.be.rejectedWith('Invalid status \"asdadsa\"');
            // @ts-ignore
            yield expect(SalesForcer.getBD2OrdersByStatus(null, {})).to.be.rejectedWith('Invalid status \"null\"');
        }));
    }));
    context('Test order search by order id/number (internal and external)', () => __awaiter(void 0, void 0, void 0, function* () {
        it('Throws an exception for invalid internal/external order number values', () => __awaiter(void 0, void 0, void 0, function* () {
            // @ts-ignore
            yield expect(SalesForcer.getBD2Orders({ yert: 1233 })).to.be.rejectedWith('Unsupported filter yert for query type BD2_Order__c');
        }));
        it('Returns null for no order found by internal/external order number', () => __awaiter(void 0, void 0, void 0, function* () {
            let noMatch = yield SalesForcer.getBD2Orders({ internalOrderNumber: "omgBBQ" });
            expect(noMatch, "No match iternal ID case failed").to.be.empty;
        }));
        it('Retrieves orders by internal order number', () => __awaiter(void 0, void 0, void 0, function* () {
            let internalOrder = yield SalesForcer.getBD2Orders({ externalOrderNumber: "SomeCustomerPO" });
            expect(internalOrder[0].details).to.deep.equal(testOrderCases.getOrderExternalId.resp[0]);
        }));
        it('Retrieves orders by external order number', () => __awaiter(void 0, void 0, void 0, function* () {
            let externalOrder = yield SalesForcer.getBD2Orders({ internalOrderNumber: "BD2-123456" });
            expect(externalOrder[0].details, "Internal ID case failed").to.deep.equal(testOrderCases.getOrderInternalId.resp[0]);
        }));
        it('Returns an order by ID', () => __awaiter(void 0, void 0, void 0, function* () {
            let idORder = yield SalesForcer.getBD2Orders({ id: "SomeId123" });
            expect(idORder[0].details, "Internal ID case failed").to.deep.equal(testOrderCases.getOrderInternalId.resp[0]);
        }));
    }));
    context('Test creating orders for locations', () => __awaiter(void 0, void 0, void 0, function* () {
        before(() => __awaiter(void 0, void 0, void 0, function* () {
            fetchMock.restore();
            fetchMock.get(testAccountCases.getAccountNoMatch.req.url, testAccountCases.getAccountNoMatch.req.body);
            fetchMock.post(/BD2_Order__c\/$/i, (url, opts) => __awaiter(void 0, void 0, void 0, function* () {
                return testOrderCases.checkOrderBody(url, opts);
            }));
        }));
        const sampleOrderObj = {
            externalOrderNumber: "asdf1321",
            orderDetails: {
                orderType: "Replacement",
                bd2Model: "Standard"
            },
            fulfillment: {
                orderStatus: "Ordered",
                qtyRequested: "5",
                shippingService: "Priority"
            },
            invoicing: {
                invoiceStatus: "QQQ"
            }
        };
        it('Validates inputs', () => __awaiter(void 0, void 0, void 0, function* () {
            let parentLocation = locationObj.createNew();
            parentLocation.loadedFromSF = true;
            parentLocation.details.id = "somePlace";
            let newOrder = parentLocation.createOrder();
            newOrder.details.externalOrderNumber = sampleOrderObj.externalOrderNumber;
            newOrder.details.orderDetails = sampleOrderObj.orderDetails;
            newOrder.details.fulfillment = sampleOrderObj.fulfillment;
            newOrder.details.invoicing = sampleOrderObj.invoicing;
            yield expect(SalesForcer.writeFile(newOrder)).to.be.rejectedWith('Field qtyRequested was type "string" but must be number');
            newOrder.details.fulfillment.qtyRequested = 5;
            yield expect(SalesForcer.writeFile(newOrder)).to.be.rejectedWith(('Field "invoiceStatus" (QQQ) is value restricted'));
            newOrder.details.invoicing.invoiceStatus = "NA";
            newOrder.details.money.moneyTotal = 100;
            newOrder.details.money.moneyCurrency = null;
            yield expect(SalesForcer.writeFile(newOrder)).to.be.rejectedWith('Can not create orders with money values and no currency');
            newOrder.details.money.moneyTotal = null;
        }));
        it('Creates a new order', () => __awaiter(void 0, void 0, void 0, function* () {
            let newOrder = orderObject.createNew();
            newOrder.details.locationId = "allGoodBro";
            newOrder.details.externalOrderNumber = sampleOrderObj.externalOrderNumber;
            newOrder.details.orderDetails = sampleOrderObj.orderDetails;
            newOrder.details.fulfillment = sampleOrderObj.fulfillment;
            newOrder.details.invoicing = sampleOrderObj.invoicing;
            newOrder.details.money.moneyCurrency = "CAD";
            newOrder.details.invoicing.invoiceStatus = "NA";
            newOrder = yield SalesForcer.writeFile(newOrder);
            expect(newOrder.details.id).to.equal('someNewOrderId');
        }));
    }));
    context('Test updating orders', () => __awaiter(void 0, void 0, void 0, function* () {
        before(() => __awaiter(void 0, void 0, void 0, function* () {
            fetchMock.restore();
            fetchMock.patch(/BD2_Order__c\/\w+?$/i, (url, opts) => __awaiter(void 0, void 0, void 0, function* () {
                return testOrderCases.checkOrderBody(url, opts);
            }));
        }));
        const sampleOrderObj = {
            externalOrderNumber: "asdf1321",
            orderDetails: {
                orderType: "Replacement",
                bd2Model: "Standard"
            },
            fulfillment: {
                orderStatus: "Ordered",
                qtyRequested: 5,
                shippingService: "Priority"
            },
            invoicing: {
                invoiceStatus: "NA"
            }
        };
        it('Rejects writes with mandatory fields with missing Id', () => __awaiter(void 0, void 0, void 0, function* () {
            let existingOrder = orderObject.createNew();
            existingOrder.loadedFromSF = true;
            //@ts-ignore
            existingOrder.details = sampleOrderObj;
            yield expect(SalesForcer.writeFile(existingOrder)).to.be.rejectedWith("Can't update a field without an Id");
        }));
        it('Rejects writes with mandatory fields with bad values', () => __awaiter(void 0, void 0, void 0, function* () {
            let existingOrder = orderObject.createNew();
            existingOrder.loadedFromSF = true;
            //@ts-ignore
            existingOrder.details = sampleOrderObj;
            existingOrder.details.id = "someOrderIdToUpdate";
            //@ts-ignore
            existingOrder.details.invoicing.invoiceStatus = "QQQQQ";
            yield expect(SalesForcer.writeFile(existingOrder)).to.be.rejectedWith('Field "invoiceStatus" (QQQQQ) is value restricted and must be one of NA or Not Sent or Sent or Shopify');
        }));
        it('Allows updates with missing fields (invoice status)', () => __awaiter(void 0, void 0, void 0, function* () {
            let existingOrder = orderObject.createNew();
            existingOrder.loadedFromSF = true;
            //@ts-ignore
            existingOrder.details = sampleOrderObj;
            //@ts-ignore
            delete existingOrder.details.invoicing.invoiceStatus;
            yield expect(SalesForcer.writeFile(existingOrder)).not.to.be.rejected;
        }));
        it('Throws exception trying to update with bad locationId', () => __awaiter(void 0, void 0, void 0, function* () {
            let existingOrder = orderObject.createNew();
            existingOrder.loadedFromSF = true;
            //@ts-ignore
            existingOrder.details = sampleOrderObj;
            existingOrder.details.id = "dontExistBy";
            yield expect(SalesForcer.writeFile(existingOrder)).to.be.rejectedWith('Invalid order parent provided');
            existingOrder.details.id = "malformedId";
            yield expect(SalesForcer.writeFile(existingOrder)).to.be.rejectedWith('Invalid order parent provided');
        }));
        it('Throws an exception for duplicate values (SF rejection)', () => __awaiter(void 0, void 0, void 0, function* () {
            let existingOrder = orderObject.createNew();
            existingOrder.loadedFromSF = true;
            //@ts-ignore
            existingOrder.details = sampleOrderObj;
            existingOrder.details.id = "duplicateId";
            yield expect(SalesForcer.writeFile(existingOrder)).to.be.rejectedWith('SF rejected write due to duplicate value on field "Name"');
        }));
        it('Throws an exception for missing mandatory values (SF rejection)', () => __awaiter(void 0, void 0, void 0, function* () {
            let existingOrder = orderObject.createNew();
            existingOrder.loadedFromSF = true;
            //@ts-ignore
            existingOrder.details = sampleOrderObj;
            existingOrder.details.id = "updateMissingFields";
            yield expect(SalesForcer.writeFile(existingOrder)).to.be.rejectedWith("Missing fields: BLAH BLAH | I'M MISSING");
        }));
    }));
    context('Testing order search by account', () => __awaiter(void 0, void 0, void 0, function* () {
        before(() => __awaiter(void 0, void 0, void 0, function* () {
            fetchMock.restore();
            fetchMock.get(testAccountCases.getAccountNoMatch.req.url, testAccountCases.getAccountNoMatch.req.body);
            fetchMock.get(testAccountCases.accountByNameOrIDWithLocations.req.url, testAccountCases.accountByNameOrIDWithLocations.req.body);
            fetchMock.get(testAccountCases.accountWithChildOrders.req.url, testAccountCases.accountWithChildOrders.req.body);
            fetchMock.get(testOrderCases.orderWhatBelongsToAccount.req.url, testOrderCases.orderWhatBelongsToAccount.req.body);
        }));
        it('Returns object with parentId set but results empty when no orders for valid parent', () => __awaiter(void 0, void 0, void 0, function* () {
            fetchMock.get(testOrderCases.noOrdersForParentAccount.req.url, testOrderCases.noOrdersForParentAccount.req.body);
            let noOrderParent = yield SalesForcer.getBD2Accounts({ accountName: 'ACME' });
            let noOrders = yield noOrderParent[0].getOrders({}, "new");
            expect(noOrders).to.be.empty;
        }));
        it('Gets orders for parents', () => __awaiter(void 0, void 0, void 0, function* () {
            let accountWithOrder = yield SalesForcer.getBD2Accounts({ accountName: "iHaveKids" }); //   (, "name", "paused")
            let ordersForThatGuy = yield accountWithOrder[0].getOrders({}, "paused");
            expect(ordersForThatGuy.map((a) => (a.details))).to.deep.equal(testAccountCases.accountWithChildOrders.resp);
            let anotherAccountWithOrder = yield SalesForcer.getBD2Accounts({ id: "someIdWithKids" }); //   (, "name", "paused")
            let ordersForThatSecondGuy = yield anotherAccountWithOrder[0].getOrders({}, "paused");
            expect(ordersForThatSecondGuy.map((a) => (a.details))).to.deep.equal(testAccountCases.accountWithChildOrders.resp);
        }));
    }));
    context('Testing order search by location', () => __awaiter(void 0, void 0, void 0, function* () {
        before(() => __awaiter(void 0, void 0, void 0, function* () {
            fetchMock.restore();
            fetchMock.get(testAccountCases.getAccountNoMatch.req.url, testAccountCases.getAccountNoMatch.req.body);
            fetchMock.get(testAccountCases.accountByNameOrIDWithLocations.req.url, testAccountCases.accountByNameOrIDWithLocations.req.body);
            fetchMock.get(testLocationCases.locationThatExists.req.url, testLocationCases.locationThatExists.req.body);
            fetchMock.get(testOrderCases.noOrdersForParentAccount.req.url, testOrderCases.noOrdersForParentAccount.req.body);
            fetchMock.get(testOrderCases.getOrderInternalId.req.url, testOrderCases.getOrderInternalId.req.body);
        }));
        it('Returns all orders for a location', () => __awaiter(void 0, void 0, void 0, function* () {
            let location = yield SalesForcer.getBD2Locations({ id: "ARealIdThatExists" });
            let orders = yield location[0].getOrders({}, "new");
            expect(orders.map((a) => (a.details))).to.deep.equal(testLocationCases.locationThatExists.orderResp);
        }));
    }));
    context('Testing order search by inventory', () => __awaiter(void 0, void 0, void 0, function* () {
        before(() => __awaiter(void 0, void 0, void 0, function* () {
            fetchMock.restore();
            fetchMock.get(testInventoryCases.noInventory.req.url, testInventoryCases.noInventory.req.body);
            fetchMock.get(testInventoryCases.inventoryById.req.url, testInventoryCases.inventoryById.req.body);
            fetchMock.get(testOrderCases.noOrdersFound.req.url, testOrderCases.noOrdersFound.req.body);
            fetchMock.get(testInventoryCases.singleInventoryforId.req.url, testInventoryCases.singleInventoryforId.req.body);
            fetchMock.get(testOrderCases.paginationTestCase.responseOne.url, testOrderCases.paginationTestCase.responseOne.body);
            fetchMock.get(testOrderCases.paginationTestCase.responseTwo.url, testOrderCases.paginationTestCase.responseTwo.body);
            fetchMock.get(testOrderCases.paginationTestCase.responseThree.url, testOrderCases.paginationTestCase.responseThree.body);
        }));
        it('Returns obj id and empty arrat for valid inv no orders', () => __awaiter(void 0, void 0, void 0, function* () {
            let noOrderInventory = yield SalesForcer.getBD2Inventories({ id: 'aResultById' });
            let ordersForInventory = yield noOrderInventory[0].getOrders({}, "new");
            expect(ordersForInventory).to.be.empty;
        }));
        it('Gets orders from a specific inventory by name and ID', () => __awaiter(void 0, void 0, void 0, function* () {
            let OrderInventory = yield SalesForcer.getBD2Inventories({ id: 'invWithOrders' });
            let ordersForInventory = yield OrderInventory[0].getOrders({}, "new");
            expect(ordersForInventory.map((a) => (a.details))).to.deep.equal(testOrderCases.paginationTestCase.expectedResults);
        }));
    }));
}));
