var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import chai from 'chai';
import chaiAsPromised from "chai-as-promised";
import * as testOrderCases from './testDetails/salesforce.cases.orders.js';
import * as testQueryCases from './testDetails/salesforce.cases.queries.js';
import * as functionalAddresses from './testDetails/salesforce.cases.functional.locations.js';
import * as SalesForcer from '../salesforce.js';
import * as sfUtilities from '../salesforceUtilities.js';
import * as orderObject from '../sfObjects/order.js';
import * as accountObject from '../sfObjects/account.js';
import * as locationObject from '../sfObjects/location.js';
import * as returnObject from '../sfObjects/return.js';
import * as inventoryObject from '../sfObjects/inventory.js';
const expect = chai.expect;
chai.use(chaiAsPromised);
describe('Test General SalesForcer Stuffs', () => __awaiter(void 0, void 0, void 0, function* () {
    context('Test query generator', () => __awaiter(void 0, void 0, void 0, function* () {
        it('Throws an error for invalid query, request, and data types', () => {
            //@ts-ignore
            expect(() => { sfUtilities.queryGenerator('dqdwdsa', {}, {}); }, 'Didnt throw error for unsupported query type').to.throw('Invalid targetType provded to queryGenerator');
            //@ts-ignore
            expect(() => { sfUtilities.queryGenerator(orderObject, { someRandomThing: 'Some Random Stuff' }); }, 'Didnt throw error for unsupported filter type').to.throw('Unsupported filter');
        });
        it('Generates query strings properly', () => {
            for (const [targetName, testCases] of Object.entries(testQueryCases.queryGeneratorTestCases)) {
                for (const testCase of testCases) {
                    let targetObject;
                    switch (targetName) {
                        case "order":
                            targetObject = orderObject;
                            break;
                        case "account":
                            targetObject = accountObject;
                            break;
                        case "location":
                            targetObject = locationObject;
                            break;
                        case "inventory":
                            targetObject = inventoryObject;
                            break;
                        case "return":
                            targetObject = returnObject;
                            break;
                    }
                    const generatedQuery = sfUtilities.queryGenerator(targetObject, testCase.filters, testCase.requiredData);
                    expect(generatedQuery).to.equal(testCase.expected);
                }
            }
        });
    }));
    context('Test SF -> Nice conversion', () => __awaiter(void 0, void 0, void 0, function* () {
        it('Works', () => __awaiter(void 0, void 0, void 0, function* () {
            //@ts-ignore
            let order = orderObject.createNew();
            order.loadFromSF(testOrderCases.getOrderExternalId.req.body.records[0]);
            let generated = order.details;
            let expected = testOrderCases.getOrderExternalId.resp[0];
            expect(generated).to.deep.equal(expected);
        }));
    }));
    context('Test Nice -> SF conversion', () => __awaiter(void 0, void 0, void 0, function* () {
        it('Validates Input', () => __awaiter(void 0, void 0, void 0, function* () {
            let order = orderObject.createNew();
            order.details = testOrderCases.getOrderExternalId.resp[0];
            //@ts-ignore
            order.details.fffffff = 12345;
            expect(() => { order.createSF(); }).to.throw('Unknown niceName \"fffffff\" for object');
            //@ts-ignore
            delete order.details.fffffff;
            //@ts-ignore
            order.details.orderDetails.dealerId = 12345;
            expect(() => { order.createSF(); }).to.throw('Field dealerId was type \"number\" but must be string or null');
            order.details.orderDetails.dealerId = "12345";
            //@ts-ignore
            order.details.omgwtfbbq = {
                invoiceStatus: "NA"
            };
            expect(() => { order.createSF(); }).to.throw('Level omgwtfbbq is not defined for object');
            //@ts-ignore
            delete order.details.omgwtfbbq;
            //@ts-ignore
            order.details.orderDetails.invoiceStatus = "NA";
            expect(() => { order.createSF(); }).to.throw('Key "invoiceStatus" is not defined for object at level "orderDetails"');
            //@ts-ignore
            delete order.details.orderDetails.invoiceStatus;
            order.details.invoicing.invoiceStatus = "ASDSAFASFQA";
            expect(() => { order.createSF(); }).to.throw('Field \"invoiceStatus\" (ASDSAFASFQA) is value restricted and must be one of NA or Not Sent or Sent or Shopify');
        }));
        it('Works too', () => __awaiter(void 0, void 0, void 0, function* () {
            testOrderCases.getOrderExternalId.resp[0].invoicing.invoiceStatus = "NA";
            testOrderCases.getOrderExternalId.resp[0].orderDetails.dealerId = null;
            let order = orderObject.createNew();
            order.details = testOrderCases.getOrderExternalId.resp[0];
            let generated = order.createSF();
            let expected = testOrderCases.getOrderExternalId.sfObject;
            expect(generated).to.deep.equal(expected);
        }));
    }));
    context('Check general write validations', () => __awaiter(void 0, void 0, void 0, function* () {
        it('Throws an exception trying to write a nonsense object', () => __awaiter(void 0, void 0, void 0, function* () {
            //@ts-ignore
            yield expect(SalesForcer.writeFile({ key: "value" })).to.be.rejectedWith('Invalid targetType provded to writeFile');
        }));
        it('Throws an exception trying to write a nonsense object', () => __awaiter(void 0, void 0, void 0, function* () {
            let newAccount = accountObject.createNew();
            //@ts-ignore
            yield expect(SalesForcer.writeFile(newAccount)).to.be.rejectedWith('Object type account is not writeable');
        }));
    }));
    context('Check address matching function', () => __awaiter(void 0, void 0, void 0, function* () {
        it(`Matches ${functionalAddresses.addressMatchingCases.length} test case streets successfully`, () => __awaiter(void 0, void 0, void 0, function* () {
            for (const testAddress of functionalAddresses.addressMatchingCases) {
                for (const variantAddress of testAddress.variants) {
                    const errString = `Expected\n\t${variantAddress.street}\nto match\n\t${testAddress.parent.street}`;
                    expect(sfUtilities.compareStreets(variantAddress.street, testAddress.parent.street), errString).to.be.true;
                }
            }
        }));
    }));
}));
